<?php

/**
 * STINA Crystal LIFT Meta Functions
 *
 * Functions related to campaign meta information.
 * Available on both the front-end and admin.
 */

use STINA\CrystalLift\Models\Resources\CampaignMeta as CampaignMetaResourceModel;
use STINA\CrystalLift\Repositories\Resources\CampaignMeta as CampaignMetaResourceRepository;

defined('ABSPATH') || exit;

/**
 * Adds a new entry to the campaign meta table.
 *
 * @param int    $campaignUniqueId Campaign Unique ID
 * @param string $metaKey   Meta key to add
 * @param mixed  $metaValue Meta value to add
 *
 * @return mixed|bool
 */
function crysli_add_campaign_meta($campaignUniqueId, $metaKey, $metaValue)
{
    /** @var CampaignMetaResourceModel $campaignMeta */
    $campaignMeta = CampaignMetaResourceRepository::instance()->insert(
        array(
            'campaign_unique_id' => $campaignUniqueId,
            'meta_key'           => $metaKey,
            'meta_value'         => maybe_serialize($metaValue)
        )
    );

    if (!$campaignMeta) {
        return false;
    }

    return $campaignMeta->getMetaValue();
}

/**
 * Retrieves one or multiple campaign meta values
 *
 * @param int    $campaignUniqueId Campaign Unique ID
 * @param string $metaKey          Meta key to search by
 * @param bool   $single           Return a single or multiple rows (if found)
 *
 * @return mixed|mixed[]|bool
 */
function crysli_get_campaign_meta($campaignUniqueId, $metaKey, $single = false)
{
    if ($single) {
        /** @var CampaignMetaResourceModel $campaignMeta */
        $campaignMeta = CampaignMetaResourceRepository::instance()->findBy(
            array(
                'campaign_unique_id' => $campaignUniqueId,
                'meta_key'           => $metaKey
            )
        );

        if (!$campaignMeta) {
            return false;
        }

        return $campaignMeta->getMetaValue();
    }

    $campaignMetas = CampaignMetaResourceRepository::instance()->findAllBy(
        array(
            'campaign_unique_id' => $campaignUniqueId,
            'meta_key'           => $metaKey
        )
    );

    $result = array();

    /** @var CampaignMetaResourceModel $licenseMeta */
    foreach ($campaignMetas as $campaignMeta) {
        $result[] = $campaignMeta->getMetaValue();
    }

    return $result;
}

/**
 * Updates existing campaign meta entries.
 *
 * @param int    $campaignUniqueId
 * @param string $metaKey
 * @param mixed  $metaValue
 * @param mixed  $previousValue
 *
 * @return bool
 */
function crysli_update_campaign_meta($campaignUniqueId, $metaKey, $metaValue, $previousValue = null)
{
    $selectQuery = array(
        'campaign_unique_id' => $campaignUniqueId,
        'meta_key'           => $metaKey
    );
    $updateQueryCondition = array(
        'campaign_unique_id' => $campaignUniqueId,
        'meta_key'           => $metaKey
    );
    $updateQueryData = array(
        'campaign_unique_id' => $campaignUniqueId,
        'meta_key'           => $metaKey,
        'meta_value'         => maybe_serialize($metaValue)
    );

    if ($previousValue !== null) {
        $selectQuery['meta_value']          = $previousValue;
        $updateQueryCondition['meta_value'] = $previousValue;
    }

    $campaignMeta = CampaignMetaResourceRepository::instance()->findBy($selectQuery);

    if (!$campaignMeta) {
        $campaignMeta = CampaignMetaResourceRepository::instance()->insert($updateQueryData);

        if ($campaignMeta) {
            return true;
        }

        return false;
    }

    $updateCount = CampaignMetaResourceRepository::instance()->updateBy($updateQueryCondition, $updateQueryData);

    if (!$updateCount) {
        return false;
    }

    return true;
}

/**
 * Deletes one or multiple rows from the campaign meta table.
 *
 * @param int    $campaignUniqueId
 * @param string $metaKey
 * @param mixed  $metaValue
 *
 * @return bool
 */
function crysli_delete_campaign_meta($campaignUniqueId, $metaKey, $metaValue = null)
{
    $deleteQueryCondition = array(
        'campaign_unique_id' => $campaignUniqueId,
        'meta_key'           => $metaKey
    );

    if ($metaValue) {
        $deleteQueryCondition['meta_value'] = $metaValue;
    }

    $deleteResult = CampaignMetaResourceRepository::instance()->deleteBy($deleteQueryCondition);

    if ($deleteResult) {
        return true;
    }

    return false;
}