<?php

use STINA\CrystalLift\Enumerators\CampaignEventTrigger;
use STINA\CrystalLift\Enumerators\CampaignEventObject;
use STINA\CrystalLift\Events\WooCommerce\CustomerRegistered;
use STINA\CrystalLift\Events\WooCommerce\OrderPlaced;
use STINA\CrystalLift\Models\Resources\CampaignRecipientRel as CampaignRecipientRelResourceModel;
use STINA\CrystalLift\Repositories\Resources\CampaignRecipientRel as CampaignRecipientRelResourceRepository;
use STINA\Library\Uuid;

defined('ABSPATH') || exit;

/**
 * Retrieve a Crystal LIFT plugin template file.
 *
 * @param string $template
 * @param array  $args
 */
function crysli_get_template($template, $args = array()) {
    stina_get_template($template, CRYSLI_PLUGIN_DIR, CRYSLI_PLUGIN_SLUG, $args);
}

/**
 * Returns a list of allowed events.
 *
 * @return array
 */
function crysli_get_events() {
    $events = array();

    // WooCommerce related event templates
    if (stina_plugin_is_active('woocommerce')) {
        $events[] = new CustomerRegistered();
        $events[] = new OrderPlaced();
    }

    return $events;
}

/**
 * Returns the variable options as <option> tags.
 *
 * @return string
 */
function crysli_get_variable_mapping_options_html() {
    $variables = crysli_get_variable_mapping_options();

    $html = '';

    foreach ($variables as $variable) {
        $html .= '<option value="' . esc_attr($variable['name']) . '">' . esc_attr($variable['label']) . '</option>';
    }

    return $html;
}

/**
 * Returns the TriggerDialog variable types as <option> tags.
 *
 * @return string
 */
function crysli_get_trigger_dialog_variable_type_options_html() {
    $fields = array(
        'integer'     => 'integer',
        'string'      => 'string',
        'boolean'     => 'boolean',
        'float'       => 'float',
        'date'        => 'date',
        'zip'         => 'zip',
        'countryCode' => 'countryCode',
        'image'       => 'image'
    );

    $html = '';

    foreach ($fields as $key => $name) {
        $html .= '<option value="' . esc_attr($key) . '">' . esc_attr($name) . '</option>';
    }

    return $html;
}

/**
 * Returns the given plugin image URL.
 *
 * @param string $image
 * @return string
 */
function crysli_get_img_url($image) {
    return CRYSLI_IMG_URL . $image;
}

/**
 * Maps variables to a given user/campaign combination.
 *
 * @param int    $userId
 * @param string $campaignUniqueId
 * @param array  $campaignVariableMapping
 * @return false|array
 */
function crysli_map_variables_to_user($userId, $campaignUniqueId, $campaignVariableMapping) {
    $mappedVariables        = array('triggerUniqueId' => Uuid::v4());
    $variableMappingOptions = crysli_get_variable_mapping_options();
    $requiredFields         = array('STREET', 'ZIP_CODE', 'CITY', 'COUNTRY');
    $variables              = array();

    foreach ($campaignVariableMapping as $mappedCampaignVariable) {
        $mappedVariable = false;

        foreach ($variableMappingOptions as $variableMappingOption) {
            // Skip if the current campaign variable is not mapped to the current option
            if ($variableMappingOption['name'] !== $mappedCampaignVariable['mapped_to']) {
                continue;
            }

            $mappedVariable = $variableMappingOption;
        }

        // Break early if the campaign variable could not be assigned to any
        // option. This should never actually happen.
        if (!$mappedVariable) {
            return false;
        }

        if ($mappedVariable['name'] === 'TEXT_FIELD') {
            $value = $mappedCampaignVariable['value'];
        } else {
            $defaultValue = get_user_meta($userId, $mappedVariable['meta'], true);

            $value = apply_filters(
                'stina_crysli_variable_mapping_values',
                ($defaultValue) ? $defaultValue : '',
                $userId,
                $mappedVariable,
                $campaignUniqueId
            );
        }

        // Break early if a required variable is missing
        if ($mappedCampaignVariable['field'] !== null
            && !$value
            && in_array($mappedCampaignVariable['field'], $requiredFields)
        ) {
            return false;
        }

        $mappedVariables[$mappedCampaignVariable['name']] = $value;

        if ($mappedCampaignVariable['field'] !== null) {
            $variables[$mappedCampaignVariable['field']] = $value;
        }
    }

    // Validate the address recipient
    if (!$variables['COMPANY'] && (!$variables['FIRST_NAME'] || !$variables['LAST_NAME'])) {
        return false;
    }

    return $mappedVariables;
}

/**
 * Retrieves an array of campaign recipients for a given campaign.
 *
 * @param string $campaignUniqueId
 * @return array
 */
function crysli_get_campaign_recipients($campaignUniqueId) {
    /** @var CampaignRecipientRelResourceModel[]|false $campaignRecipientRels */
    $campaignRecipientRels = CampaignRecipientRelResourceRepository::instance()->findAllBy(
        array(
            'campaign_unique_id' => $campaignUniqueId
        )
    );

    $campaignRecipients = array();

    if ($campaignRecipientRels !== false) {
        foreach ($campaignRecipientRels as $campaignRecipientRel) {
            $campaignRecipients[] = $campaignRecipientRel->toArray();
        }
    }

    return $campaignRecipients;
}

/**
 * @param string      $dateTimeString
 * @param bool        $echo
 * @param null|string $format
 * @param null|string $dateTimeZone
 * @return void|DateTime
 * @throws Exception
 */
function crysli_get_timestamp_from_utc($dateTimeString, $echo = false, $format = null, $dateTimeZone = null) {
    if ($format === null) {
        $dateFormat = get_option('date_format');
        $timeFormat = get_option('time_format');

        $format = $dateFormat . ' ' . $timeFormat;
    }

    if ($dateTimeZone === null && $dateTimeZoneString = get_option('timezone_string', null)) {
        $dateTimeZone = new DateTimeZone($dateTimeZoneString);
    }

    $dateTime = new DateTime($dateTimeString, $dateTimeZone);

    if ($echo === false) {
        return $dateTime;
    }

    echo $dateTime->format($format);
}

/**
 * Retrieves a list of allowed events for the given object.
 *
 * @param string $object
 * @return array
 */
function crysli_get_object_events($object) {
    switch ($object) {
        case CampaignEventObject::CUSTOMER:
            $events = array(
                array(
                    'key'         => CampaignEventTrigger::REGISTERED,
                    'value'       => __('Registered', 'stina-crystal-lift'),
                    'description' => _x(
                        'Triggers when a new customer registers on the store.',
                        'Refers to an event on the CUSTOMER object.',
                        'stina-crystal-lift'
                    )
                )
            );
            break;
        case CampaignEventObject::ORDER:
            $events = array(
                array(
                    'key'         => CampaignEventTrigger::PLACED,
                    'value'       => __('Placed', 'stina-crystal-lift'),
                    'description' => _x(
                        'Triggers whenever an order is placed.',
                        'Refers to an event on the ORDER object.',
                        'stina-crystal-lift'
                    )
                )
            );
            break;
        default:
            $events = array();
            break;
    }

    return $events;
}

/**
 * Retrieves a list of allowed conditions for the given object/event combination
 *
 * @param string $object
 * @param string $event
 * @return array
 */
function crysli_get_event_conditions($object, $event) {
    $billingCountry = array(
        'entity' => array(
            'value' => 'billing_country',
            'label' => __('Billing country', 'stina-crystal-lift')
        ),
        'operators' => array('IN', 'NIN'),
        'hook'      => 'crysli_filter_values_billing_country'
    );
    $shippingCountry = array(
        'entity' => array(
            'value' => 'shipping_country',
            'label' => __('Shipping country', 'stina-crystal-lift')
        ),
        'operators' => array('IN', 'NIN'),
        'hook'      => 'crysli_filter_values_shipping_country'
    );
    $product = array(
        'entity' => array(
            'value' => 'product',
            'label' => __('Product', 'stina-crystal-lift')
        ),
        'operators' => array('IN', 'NIN'),
        'hook'      => 'crysli_filter_values_product'
    );

    $conditions = array();

    switch ($object) {
        case CampaignEventObject::ORDER:
            switch ($event) {
                case CampaignEventTrigger::PLACED:
                    $conditions = array($billingCountry, $shippingCountry, $product);
                    break;
            }
            break;
        case CampaignEventObject::CUSTOMER:
            switch ($event) {
                case CampaignEventTrigger::REGISTERED:
                    $conditions = array($billingCountry, $shippingCountry);
                    break;
            }
            break;
    }

    return $conditions;
}

/**
 * Returns the retrieved value for the given entity/args combination.
 *
 * @param string $entity
 * @param array  $args
 * @return array|bool
 */
function crysli_get_condition_entity_value($entity, $args) {
    switch ($entity) {
        case 'billing_country':
            if (!isset($args['user_id'])) {
                $order = wc_get_order(intval($args['order_id']));
                $userId = $order->get_customer_id();
            } else {
                $userId = intval($args['user_id']);
            }

            if (!$billingCountry = get_user_meta($userId, 'billing_country', true)) {
                return false;
            }

            return (array)$billingCountry;
        case 'shipping_country':
            if (!isset($args['user_id'])) {
                $order = wc_get_order($args['order_id']);
                $userId = $order->get_customer_id();
            } else {
                $userId = $args['user_id'];
            }


            if (!$shippingCountry = get_user_meta($userId, 'shipping_country', true)) {
                return false;
            }

            return (array)$shippingCountry;
        case 'product':
            $toCompare = array();
            $order = wc_get_order($args['order_id']);

            /** @var WC_Order_Item_Product $orderItemProduct */
            foreach ($order->get_items() as $orderItemProduct) {
                $orderedProductId = ($orderItemProduct->get_variation_id())
                    ? $orderItemProduct->get_variation_id()
                    : $orderItemProduct->get_product_id();

                $toCompare[] = $orderedProductId;
            }

            return $toCompare;
    }

    return false;
}

/**
 * Returns the event uuid by an object/trigger combination.
 *
 * @param $object
 * @param $trigger
 * @return null|string
 */
function crysli_get_event_unique_id($object, $trigger) {
    $eventUniqueId = null;

    switch ($object) {
        case CampaignEventObject::CUSTOMER:
            switch ($trigger) {
                case CampaignEventTrigger::REGISTERED:
                    $eventUniqueId = CustomerRegistered::UNIQUE_ID;
                    break;
            }
            break;
        case CampaignEventObject::ORDER:
            switch ($trigger) {
                case CampaignEventTrigger::PLACED:
                    $eventUniqueId = OrderPlaced::UNIQUE_ID;
                    break;
            }
            break;
    }

    return $eventUniqueId;
}

/**
 * Returns the available variable mapping options.
 *
 * @return array
 */
function crysli_get_variable_mapping_options() {
    $options = array(
        array(
            'name'  => 'WP_FIRST_NAME',
            'label' => __('WordPress User Profile - First name', 'stina-crystal-lift'),
            'meta'  => 'first_name'
        ),
        array(
            'name'  => 'WP_LAST_NAME',
            'label' => __('WordPress User Profile - Last name', 'stina-crystal-lift'),
            'meta'  => 'last_name'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_FIRST_NAME',
            'label' => __('WooCommerce Shipping Address - First name', 'stina-crystal-lift'),
            'meta'  => 'shipping_first_name'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_LAST_NAME',
            'label' => __('WooCommerce Shipping Address - Last name', 'stina-crystal-lift'),
            'meta'  => 'shipping_last_name'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_COMPANY',
            'label' => __('WooCommerce Shipping Address - Company', 'stina-crystal-lift'),
            'meta'  => 'shipping_company'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_ADDRESS_LINE_1',
            'label' => __('WooCommerce Shipping Address - Address Line 1', 'stina-crystal-lift'),
            'meta'  => 'shipping_address_1'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_ADDRESS_LINE_2',
            'label' => __('WooCommerce Shipping Address - Address Line 2', 'stina-crystal-lift'),
            'meta'  => 'shipping_address_2'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_CITY',
            'label' => __('WooCommerce Shipping Address - City', 'stina-crystal-lift'),
            'meta'  => 'shipping_city'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_POSTCODE',
            'label' => __('WooCommerce Shipping Address - Postcode/ZIP', 'stina-crystal-lift'),
            'meta'  => 'shipping_postcode'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_COUNTRY',
            'label' => __('WooCommerce Shipping Address - Country (ISO2)', 'stina-crystal-lift'),
            'meta'  => 'shipping_country'
        ),
        array(
            'name'  => 'WC_SHIPPING_ADDRESS_STATE',
            'label' => __('WooCommerce Shipping Address - State', 'stina-crystal-lift'),
            'meta'  => 'shipping_state'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_COMPANY',
            'label' => __('WooCommerce Billing Address - Company', 'stina-crystal-lift'),
            'meta'  => 'billing_first_name'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_FIRST_NAME',
            'label' => __('WooCommerce Billing Address - First name', 'stina-crystal-lift'),
            'meta'  => 'billing_last_name'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_LAST_NAME',
            'label' => __('WooCommerce Billing Address - Last name', 'stina-crystal-lift'),
            'meta'  => 'billing_company'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_ADDRESS_LINE_1',
            'label' => __('WooCommerce Billing Address - Address Line 1', 'stina-crystal-lift'),
            'meta'  => 'billing_address_1'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_ADDRESS_LINE_2',
            'label' => __('WooCommerce Billing Address - Address Line 2', 'stina-crystal-lift'),
            'meta'  => 'billing_address_2'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_CITY',
            'label' => __('WooCommerce Billing Address - City', 'stina-crystal-lift'),
            'meta'  => 'billing_city'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_POSTCODE',
            'label' => __('WooCommerce Billing Address - Postcode/ZIP', 'stina-crystal-lift'),
            'meta'  => 'billing_postcode'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_COUNTRY',
            'label' => __('WooCommerce Billing Address - Country (ISO2)', 'stina-crystal-lift'),
            'meta'  => 'billing_country'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_STATE',
            'label' => __('WooCommerce Billing Address - State', 'stina-crystal-lift'),
            'meta'  => 'billing_state'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_PHONE',
            'label' => __('WooCommerce Billing Address - Phone', 'stina-crystal-lift'),
            'meta'  => 'billing_phone'
        ),
        array(
            'name'  => 'WC_BILLING_ADDRESS_EMAIL',
            'label' => __('WooCommerce Billing Address - Email', 'stina-crystal-lift'),
            'meta'  => 'billing_email'
        ),
        array(
            'name'  => 'TEXT_FIELD',
            'label' => __('Text input field', 'stina-crystal-lift'),
            'meta'  => null
        )
    );

    return apply_filters('stina_crysli_variable_mapping_options', $options);
}

function crysli_get_logical_operators($selected = null) {
    $operators = array(
        'LT'  => __('Less than', 'stina-crystal-lift'),
        'LE'  => __('Less than or equal to', 'stina-crystal-lift'),
        'GT'  => __('Greater than', 'stina-crystal-lift'),
        'GE'  => __('Greater than or equal', 'stina-crystal-lift'),
        'EQ'  => __('Equals to', 'stina-crystal-lift'),
        'IN'  => __('Is one of', 'stina-crystal-lift'),
        'NIN' => __('Is none of', 'stina-crystal-lift')
    );

    if ($selected !== null && is_array($selected)) {
        foreach ($operators as $key => $value) {
            if (array_search($key, $selected) === false) {
                unset($operators[$key]);
            }
        }
    }

    return $operators;
}

function crysli_get_filter_fields($field) {
    $fields = array(
        'DATE'   => crysli_get_logical_operators(array('LT', 'GT')),
        'MULTI'  => crysli_get_logical_operators(array('IN', 'NIN')),
        'NUMBER' => crysli_get_logical_operators(array('LT', 'LE', 'GT', 'GE', 'EQ'))
    );

    if (!isset($fields[$field])) {
        return null;
    }

    return $fields[$field];
}

function crysli_get_filter_objects() {
    return array(
        array(
            'name'   => 'customer',
            'label'  => __('Customer', 'stina-crystal-lift'),
            'values' => array(
                array(
                    'name'   => 'registration_date',
                    'label'  => __('Registration date', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'DATE',
                            'name'   => 'date',
                            'label'  => __('Date', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('DATE')
                        )
                    )
                ),
                array(
                    'name'   => 'language',
                    'label'  => __('Language', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'MULTI',
                            'name'   => 'language',
                            'label'  => __('Language', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('MULTI')
                        )
                    )
                ),
                array(
                    'name'   => 'shipping_country',
                    'label'  => __('Shipping country', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'MULTI',
                            'name'   => 'shipping_country',
                            'label'  => __('Shipping country', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('MULTI')
                        )
                    )
                ),
                array(
                    'name'   => 'billing_country',
                    'label'  => __('Billing country', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'MULTI',
                            'name'   => 'billing_country',
                            'label'  => __('Billing country', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('MULTI')
                        )
                    )
                )
            )
        ),
        array(
            'name'   => 'order',
            'label'  => __('Order', 'stina-crystal-lift'),
            'values' => array(
                array(
                    'name'   => 'first_order_date',
                    'label'  => __('First order date', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'DATE',
                            'name'   => 'first_order_date',
                            'label'  => __('First order date', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('DATE')
                        )
                    )
                ),
                array(
                    'name'   => 'last_order_date',
                    'label'  => __('Last order date', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'DATE',
                            'name'   => 'last_order_date',
                            'label'  => __('Last order date', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('DATE')
                        )
                    )
                ),
                array(
                    'name'   => 'number_of_orders',
                    'label'  => __('Number of orders', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'NUMBER',
                            'name'   => 'number_of_orders',
                            'label'  => __('Number of orders', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('NUMBER')
                        )
                    )
                ),
                array(
                    'name'   => 'revenue_completed',
                    'label'  => __('Revenue (Completed orders)', 'stina-crystal-lift'),
                    'fields' => array(
                        array(
                            'type'   => 'NUMBER',
                            'name'   => 'revenue',
                            'label'  => __('Revenue', 'stina-crystal-lift'),
                            'values' => crysli_get_filter_fields('NUMBER')
                        )
                    )
                )
            )
        )
    );
}
