<?php


namespace STINA\CrystalLift\Components;

defined('ABSPATH') || exit;

use DateTime;
use Exception;
use GuzzleHttp\Exception\GuzzleException;
use STINA\CrystalLift\Abstracts\Event;
use STINA\CrystalLift\Enumerators\CampaignAction as CampaignActionEnumerator;
use STINA\CrystalLift\Enumerators\CampaignAutomation as CampaignAutomationEnumerator;
use STINA\CrystalLift\Enumerators\CampaignProvider as CampaignProviderEnumerator;
use STINA\CrystalLift\Enumerators\CampaignStatus as CampaignStatusEnumerator;
use STINA\CrystalLift\Enumerators\CampaignType as CampaignTypeEnumerator;
use STINA\CrystalLift\Enumerators\Feature;
use STINA\CrystalLift\Enumerators\Page as PageEnumerator;
use STINA\CrystalLift\Events\Manager as EventManager;
use STINA\CrystalLift\Models\Campaign as CampaignModel;
use STINA\CrystalLift\Models\Resources\CampaignRecipientRel as CampaignRecipientRelResourceModel;
use STINA\CrystalLift\Services\CrystalLift as CrystalLiftService;
use STINA\CrystalLift\Repositories\Resources\CampaignRecipientRel as CampaignRecipientRelResourceRepository;
use STINA\Library\Alert;
use STINA\Library\Components\UI\Heading;
use STINA\Library\Interfaces\Component as ComponentInterface;
use STINA\Library\Models\User as UserModel;
use STINA\Library\Repositories\User as UserRepository;
use WC_Order;
use WP_User;
use WP_User_Query;

class Campaigns implements ComponentInterface
{
    /**
     * @var false|UserModel
     */
    protected $user;

    /**
     * @var array
     */
    protected $ui;

    /**
     * Campaigns constructor.
     */
    public function __construct()
    {
        $this->user = UserRepository::instance()->get();

        // UI Components
        $this->ui = array(
            'heading' => new Heading(
                __('Campaigns', 'stina-crystal-lift'),
                __('Sign out', 'stina-crystal-lift'),
                $this->user,
                'crysli',
                array(
                    'redirect' => '/wp-admin/admin.php?page=' . PageEnumerator::DASHBOARD
                )
            )
        );

        add_action('admin_menu',                    array($this, 'init'));
        add_action('wp_ajax_crysli_campaigns_load', array($this, 'load'));

        add_action('admin_post_crysli_campaign_create_campaign',                    array($this, 'createCampaign'));
        add_action('admin_post_crysli_campaign_duplicate_campaign',                 array($this, 'duplicateCampaign'));
        add_action('admin_post_crysli_campaign_send_campaign',                      array($this, 'sendCampaign'));
        add_action('admin_post_crysli_campaign_update_action',                      array($this, 'updateCampaignAction'));
        add_action('admin_post_crysli_campaign_update_status',                      array($this, 'updateCampaignStatus'));
        add_action('admin_post_crysli_campaign_update_event',                       array($this, 'updateCampaignEvent'));
        add_action('admin_post_crysli_campaign_update_recipients',                  array($this, 'updateCampaignRecipients'));
        add_action('admin_post_crysli_campaign_update_trigger_dialog_confirmation', array($this, 'updateTriggerDialogConfirmation'));

        add_action('wp_ajax_crysli_modal_campaign_create_wizard_load',          array($this, 'campaignCreateWizardLoad'));
        add_action('wp_ajax_crysli_modal_campaign_configure_event_wizard_load', array($this, 'campaignConfigureEventWizardLoad'));
        add_action('wp_ajax_crysli_modal_campaign_edit_recipients_wizard_load', array($this, 'campaignEditRecipientsWizardLoad'));
    }

    /**
     * @inheritDoc
     */
    public function init()
    {
        if (!UserRepository::instance()->get()) {
            return;
        }

        add_submenu_page(
            'crystal-lift',
            __('Campaigns', 'stina-crystal-lift'),
            __('Campaigns', 'stina-crystal-lift'),
            'manage_options',
            PageEnumerator::CAMPAIGNS,
            array($this, 'showPage')
        );
    }

    /**
     * @inheritDoc
     */
    public function showPage()
    {
        wp_enqueue_script('stina-framework');

        if (isset($_GET['id'])) {
            $campaignUniqueId = sanitize_text_field($_GET['id']);

            if (isset($_GET['action']) && $_GET['action'] === 'view-report') {
                $this->loadComponentViewReport($campaignUniqueId);
            } else {
                $this->loadComponentEditCampaign($campaignUniqueId);
            }

        } else {
            $this->loadComponentMain();
        }
    }

    /**
     * Loads the main campaign component.
     *
     * @return void
     */
    private function loadComponentMain()
    {
        // Enqueue needed scripts and styles
        wp_enqueue_style('crysli-page-campaigns');
        wp_enqueue_style('crysli-modal-campaign-create-wizard');
        wp_enqueue_script('stina-fontawesome');
        wp_enqueue_script('stina-vue-component-pagination');
        wp_enqueue_script('crysli-component-campaigns');
        wp_enqueue_script('crysli-modal-campaign-create-wizard');

        // Localize scripts and pass needed variables
        wp_localize_script(
            'crysli-component-campaigns',
            'crysliComponentCampaignsSecurity',
            array(
                'loadCampaigns' => wp_create_nonce('crysli_campaigns_load')
            )
        );

        wp_localize_script(
            'crysli-component-campaigns',
            'i18n',
            array(
                'LABELS' => array(
                    'campaign' => array(
                        'action'     => CampaignActionEnumerator::getLabelArray(),
                        'provider'   => CampaignProviderEnumerator::getLabelArray(),
                        'type'       => CampaignTypeEnumerator::getLabelArray(),
                        'automation' => CampaignAutomationEnumerator::getLabelArray(),
                        'status'     => CampaignStatusEnumerator::getLabelArray()
                    )
                )
            )
        );

        wp_localize_script(
            'crysli-modal-campaign-create-wizard',
            'security',
            array(
                'modalCampaignWizardLoad' => wp_create_nonce('crysli_modal_campaign_create_wizard_load')
            )
        );

        // Load templates
        crysli_get_template('modal-campaign-create-wizard.php');
        crysli_get_template('dialog-campaign-duplicate.php');

        crysli_get_template(
            'page-campaigns/component-main.php',
            array(
                'ui' => $this->ui
            )
        );
    }

    /**
     * Loads the campaign report component.
     *
     * @param string $campaignUniqueId
     * @throws Exception|GuzzleException
     * @return void
     */
    private function loadComponentViewReport($campaignUniqueId)
    {
        /** @var false|array $campaign */
        $campaign = CrystalLiftService::instance()->userCampaignGetCampaign($campaignUniqueId);

        if (!$campaign) {
            Alert::danger(__('The campaign could not be found.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS));
            exit;
        }

        $filter = array(
            array(
                'key' => 'startDate',
                'value' => '2020-01-01'
            ),
            array(
                'key' => 'endDate',
                'value' => '2120-01-01'
            )
        );

        /** @var false|array $trigger */
        $trigger = CrystalLiftService::instance()->userCampaignListTrigger($campaignUniqueId, $filter);

        wp_enqueue_style('stina-framework');
        wp_enqueue_script('stina-fontawesome');

        crysli_get_template(
            'page-campaigns/component-view-report.php',
            array(
                'campaign' => $campaign['data'],
                'triggers' => ($trigger) ? $trigger['data'] : false
            )
        );
    }

    /**
     * Loads the edit campaign component.
     *
     * @param string $campaignUniqueId
     * @throws Exception|GuzzleException
     * @return void
     */
    private function loadComponentEditCampaign($campaignUniqueId)
    {
        /** @var false|array $campaign */
        $campaign = CrystalLiftService::instance()->userCampaignGetCampaign($campaignUniqueId);

        if (!$campaign) {
            Alert::danger(__('The campaign could not be found.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS));
            exit;
        }

        /** @var CampaignRecipientRelResourceModel[]|false $campaignRecipients */
        $campaignRecipients = CampaignRecipientRelResourceRepository::instance()->findAllBy(
            array(
                'campaign_unique_id' => $campaignUniqueId
            )
        );

        $campaignRecipientUserIds = array();

        if ($campaignRecipients) {
            foreach ($campaignRecipients as $campaignRecipient) {
                $campaignRecipientUserIds[] = $campaignRecipient->getUserId();
            }
        }

        // Enqueue needed scripts and styles
        wp_enqueue_style('stina-select2');
        wp_enqueue_style('crysli-page-edit-campaign');
        wp_enqueue_style('crysli-modal-campaign-edit-content-wizard');
        wp_enqueue_script('stina-fontawesome');
        wp_enqueue_script('stina-vue-component-select2');
        wp_enqueue_script('stina-vue-component-pagination');
        wp_enqueue_script('crysli-modal-campaign-edit-content-wizard');

        // Localize scripts and pass needed variables
        wp_localize_script(
            'crysli-modal-campaign-edit-content-wizard',
            'data',
            array(
                'campaign'                => $campaign['data'],
                'campaignUniqueId'        => $campaignUniqueId,
                'campaignVariableMapping' => crysli_get_campaign_meta($campaignUniqueId, 'variable_mapping', true)
            )
        );

        wp_localize_script(
            'crysli-modal-campaign-edit-content-wizard',
            'security',
            array(
                'modalCampaignEditContentWizardLoad' => wp_create_nonce('crysli_modal_campaign_edit_content_wizard_load')
            )
        );

        wp_localize_script(
            'crysli-modal-campaign-edit-content-wizard',
            'i18n',
            array(
                'variableMappingFields' => array(
                    'FIRST_NAME' => __('First name', 'stina-crystal-lift'),
                    'LAST_NAME'  => __('Last name', 'stina-crystal-lift'),
                    'COMPANY'    => __('Company', 'stina-crystal-lift'),
                    'STREET'     => __('Street', 'stina-crystal-lift'),
                    'ZIP_CODE'   => __('ZIP code', 'stina-crystal-lift'),
                    'CITY'       => __('City', 'stina-crystal-lift'),
                    'COUNTRY'    => __('Country', 'stina-crystal-lift')
                )
            )
        );

        // Enqueue and localize campaign type specific scripts and styles
        switch ($campaign['data']['campaignType']) {
            case CampaignTypeEnumerator::CAMPAIGN_REGULAR:
                if (!function_exists('wp_get_available_translations')) {
                    require_once ABSPATH . 'wp-admin/includes/translation-install.php';
                }

                $translations = wp_get_available_translations();
                $countries = array();
                $languages = array();
                $conditions = crysli_get_campaign_meta($campaignUniqueId, 'recipient_conditions', true);

                if ($conditions) {
                    foreach ($conditions as $condition) {
                        if (in_array($condition['entity'], array('shipping_country', 'billing_country'))
                            && is_array($condition['value'])
                        ) {
                            foreach ($condition['value'] as $iso2) {
                                $countries[$iso2] = WC()->countries->countries[$iso2];
                            }
                        }

                        if ($condition['entity'] === 'language' && is_array($condition['value'])) {
                            foreach ($condition['value'] as $locale) {
                                if (isset($translations[$locale])) {
                                    $languages[$locale] = $translations[$locale]['english_name'];
                                }
                            }

                        }
                    }
                }

                // Enqueue needed scripts and styles
                wp_enqueue_style('crysli-modal-campaign-edit-recipients-wizard');
                wp_enqueue_script('crysli-modal-campaign-edit-recipients-wizard');

                // Localize scripts and pass needed variables
                wp_localize_script(
                    'crysli-modal-campaign-edit-recipients-wizard',
                    'data',
                    array(
                        'campaignRecipients' => $campaignRecipientUserIds,
                        'filters'            => crysli_get_filter_objects(),
                        'conditions'         => $conditions,
                        'countries'          => $countries,
                        'languages'          => $languages
                    )
                );

                wp_localize_script(
                    'crysli-modal-campaign-edit-recipients-wizard',
                    'security',
                    array(
                        'loadSubscribers' => wp_create_nonce('crysli_modal_campaign_edit_recipients_wizard_load'),
                        'dropdownSearch'  => wp_create_nonce('crysli_modal_campaign_edit_dropdown_search')
                    )
                );

                wp_localize_script(
                    'crysli-modal-campaign-edit-recipients-wizard',
                    'i18n',
                    array(
                        'searchFor' => __('Search for', 'stina-crystal-lift'),
                        'entity' => array(
                            'language'         => __('Language', 'stina-crystal-lift'),
                            'shipping_country' => __('Country', 'stina-crystal-lift'),
                            'billing_country'  => __('Country', 'stina-crystal-lift')
                        )
                    )
                );

                break;
            case CampaignTypeEnumerator::CAMPAIGN_AUTOMATED_EVENT_BASED:
                // Enqueue needed scripts and styles
                wp_enqueue_style('crysli-modal-campaign-configure-event-wizard');
                wp_enqueue_script('crysli-modal-campaign-configure-event-wizard');

                // Localize scripts and pass needed variables
                wp_localize_script(
                    'crysli-modal-campaign-configure-event-wizard',
                    'data',
                    array(
                        'campaign' => $campaign['data']
                    )
                );

                wp_localize_script(
                    'crysli-modal-campaign-configure-event-wizard',
                    'i18n',
                    array(
                        'operators' => array(
                            'IN'  => __('Is one of', 'stina-crystal-lift'),
                            'NIN' => __('is none of', 'stina-crystal-lift')
                        ),
                        'searchFor' => __('Search for', 'stina-crystal-lift'),
                        'entity' => array(
                            'language'         => __('Language', 'stina-crystal-lift'),
                            'shipping_country' => __('Country', 'stina-crystal-lift'),
                            'billing_country'  => __('Country', 'stina-crystal-lift')
                        )
                    )
                );

                wp_localize_script(
                    'crysli-modal-campaign-configure-event-wizard',
                    'security',
                    array(
                        'modalCampaignConfigureEventWizardLoad' => wp_create_nonce('crysli_modal_campaign_configure_event_wizard_load'),
                        'getObjectEvents'    => wp_create_nonce('crysli_modal_campaign_edit_get_object_events'),
                        'getEventConditions' => wp_create_nonce('crysli_modal_campaign_edit_get_event_conditions'),
                        'dropdownSearch'     => wp_create_nonce('crysli_modal_campaign_edit_dropdown_search')
                    )
                );

                break;
        }

        // Load templates
        crysli_get_template(
            'modal-campaign-edit-content-wizard.php',
            array(
                'campaign' => $campaign['data'],
                'user'     => $this->user
            )
        );

        crysli_get_template(
            'modal-campaign-edit-recipients-wizard.php',
            array(
                'campaign' => $campaign['data'],
                'user'     => $this->user,
            )
        );

        crysli_get_template(
            'modal-campaign-configure-event-wizard.php',
            array(
                'campaign'            => $campaign['data'],
                'campaignEventConfig' => crysli_get_campaign_meta($campaignUniqueId, 'event_configuration', true),
                'user'                => $this->user
            )
        );

        crysli_get_template(
            'page-campaigns/component-edit-campaign.php',
            array(
                'campaign'                  => $campaign['data'],
                'campaignType'              => $campaign['data']['campaignType'],
                'campaignStatus'            => $campaign['data']['campaignStatus'],
                'campaignAction'            => $campaign['data']['actions'][0]['actionDescShort'],
                'campaignConfig'            => $campaign['data']['actions'][0]['config'],
                'isEventConfigured'         => EventManager::isCampaignEventConfigured($campaignUniqueId),
                'isTriggerDialogConfigured' => crysli_get_campaign_meta($campaignUniqueId, 'trigger_dialog_confirmation', true),
                'recipients'                => $campaignRecipients
            )
        );
    }

    /**
     * Loads the campaigns for the main campaign component
     *
     * @return void
     * @throws Exception
     */
    public function load()
    {
        check_ajax_referer('crysli_campaigns_load', 'security');

        $page   = intval($_POST['page']);
        $limit  = intval($_POST['limit']);
        $filter = array();
        $term   = null;

        foreach (json_decode(stripslashes($_POST['filter']), true) as $filterItem) {
            $filter[] = array(
                'key'   => sanitize_text_field($filterItem['key']),
                'value' => sanitize_text_field($filterItem['value'])
            );
        }

        if (isset($_POST['term']) && strlen($_POST['term']) > 0) {
            $term = sanitize_text_field($_POST['term']);
        }

        /** @var CampaignModel|false $campaign */
        $campaigns = CrystalLiftService::instance()->userCampaignListCampaign($page, $limit, array(), $filter, $term);

        if (!$campaigns) {
            wp_send_json(array());
            exit;
        }

        wp_send_json($campaigns);
        exit;
    }

    /**
     * Creates a new campaign
     *
     * @return void
     * @throws Exception|GuzzleException
     */
    public function createCampaign()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_create_campaign')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS));
            exit;
        }

        $nodeUniqueId          = get_option('stina_node_unique_id');
        $campaignDescLong      = sanitize_text_field($_POST['campaign_name']);
        $campaignType          = sanitize_text_field($_POST['campaign_type']);
        $campaignEventUniqueId = null;
        $feature               = Feature::CAMPAIGN_REGULAR_MANUAL;
        $campaignEvents        = array();

        $campaignActions = array(
            array(
                'actionUniqueId' => sanitize_text_field($_POST['campaign_action_unique_id'])
            )
        );

        if ($_POST['campaign_event_unique_id']) {
            $campaignEventUniqueId = sanitize_text_field($_POST['campaign_event_unique_id']);
            $campaignEvents[] = array(
                'eventUniqueId' => $campaignEventUniqueId
            );
        }

        if ($campaignType === 'CAMPAIGN_AUTOMATED_TEMPLATES') {
            $campaignType = CampaignTypeEnumerator::CAMPAIGN_AUTOMATED_EVENT_BASED;
            $feature      = Feature::CAMPAIGN_AUTO_EVENT;
        }

        /** @var false|array $campaign */
        $campaign = CrystalLiftService::instance()->userCampaignCreateCampaign(
            $feature,
            $nodeUniqueId,
            $campaignDescLong,
            $campaignType,
            null,
            $campaignEvents,
            $campaignActions
        );

        if ($campaign === false) {
            Alert::danger(__('Your campaign could not be created.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS));
            exit;
        }

        $campaignUniqueId = $campaign['data']['campaignUniqueId'];

        if ($campaignEventUniqueId && $event = EventManager::getEvent($campaignEventUniqueId)) {
            crysli_update_campaign_meta($campaignUniqueId, 'event_configuration', $event->getConfig());
        }

        Alert::success(__('Your campaign was created successfully.', 'stina-crystal-lift'));
        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $campaignUniqueId));
        exit;
    }

    /**
     * Duplicates an existing campaign.
     *
     * @return void
     * @throws GuzzleException
     */
    public function duplicateCampaign()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_duplicate_campaign')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        $oldCampaignUniqueId = sanitize_text_field($_POST['campaign_unique_id']);
        $campaignName        = sanitize_text_field($_POST['campaign_name']);
        $oldCampaign         = CrystalLiftService::instance()->userCampaignGetCampaign($oldCampaignUniqueId);

        if (!$oldCampaign) {
            Alert::danger(__('The campaign could not be found.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS));
            exit;
        }

        switch ($oldCampaign['data']['campaignType']) {
            case CampaignTypeEnumerator::CAMPAIGN_REGULAR:
                $feature = Feature::CAMPAIGN_REGULAR_MANUAL;
                break;
            case CampaignTypeEnumerator::CAMPAIGN_AUTOMATED_EVENT_BASED:
                $feature = Feature::CAMPAIGN_AUTO_EVENT;
                break;
            default:
                $feature = Feature::GENERAL;
                break;
        }

        $newCampaign = CrystalLiftService::instance()->userCampaignCreateCampaign(
            $feature,
            get_option('stina_node_unique_id', null),
            $campaignName,
            null,
            $oldCampaignUniqueId,
            array(),
            array()
        );

        if (!$newCampaign) {
            Alert::danger(__('Your campaign could not be duplicated.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS));
            exit;
        }

        $newCampaignUniqueId = $newCampaign['data']['campaignUniqueId'];
        $variableMapping     = crysli_get_campaign_meta($oldCampaignUniqueId, 'variable_mapping', true);
        $eventConfiguration  = crysli_get_campaign_meta($oldCampaignUniqueId, 'event_configuration', true);

        crysli_update_campaign_meta($newCampaignUniqueId, 'variable_mapping', $variableMapping);
        crysli_update_campaign_meta($newCampaignUniqueId, 'event_configuration', $eventConfiguration);

        if ($oldCampaign['data']['type'] === CampaignTypeEnumerator::CAMPAIGN_REGULAR) {
            $recipientConditions = crysli_get_campaign_meta($oldCampaignUniqueId, 'recipient_conditions', true);
            crysli_update_campaign_meta($oldCampaignUniqueId, 'recipient_conditions', $recipientConditions);
        }

        Alert::success(__('Your campaign was created successfully.', 'stina-crystal-lift'));
        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $newCampaignUniqueId));
        exit;
    }

    /**
     * Triggers a regular campaign
     *
     * @return void
     * @throws GuzzleException
     */
    public function sendCampaign()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_send_campaign')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        $campaignUniqueId = sanitize_text_field($_POST['campaign_unique_id']);
        $userIds          = array();

        /** @var false|array $response */
        $response = CrystalLiftService::instance()->userCampaignGetCampaign($campaignUniqueId);

        if (!$response) {
            Alert::danger(__('The campaign could not be found.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS));
            exit;
        }

        $campaign = $response['data'];

        /** @var false|CampaignRecipientRelResourceModel[] $campaignRecipients */
        $campaignRecipients = CampaignRecipientRelResourceRepository::instance()->findAllBy(
            array(
                'campaign_unique_id' => $campaignUniqueId
            )
        );

        foreach ($campaignRecipients as $campaignRecipient) {
            $userIds[] = $campaignRecipient->getUserId();
        }

        EventManager::trigger($campaign, $userIds);

        Alert::success(__('Your campaign was submitted successfully.', 'stina-crystal-lift'));
        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $campaignUniqueId . '&action=view-report'));
        exit;
    }

    /**
     * Updates the campaign's action.
     *
     * @return void
     * @throws Exception
     */
    public function updateCampaignAction()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_update_action')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        $campaignUniqueId  = sanitize_text_field($_POST['campaign_id']);
        $campaignName      = sanitize_text_field($_POST['campaign_name']);
        $variables         = array();
        $variableMapping   = array();

        // Prepare the campaign variables
        foreach ($_POST['campaign_variable'] as $variable) {
            $field = (isset($variable['field'])) ? sanitize_text_field($variable['field']) : null;

            $variables[] = array(
                'name'  => sanitize_text_field($variable['name']),
                'type'  => sanitize_text_field($variable['type']),
                'field' => $field
            );

            $mappedVariable = array(
                'name'      => sanitize_text_field($variable['name']),
                'type'      => sanitize_text_field($variable['type']),
                'mapped_to' => sanitize_text_field($variable['mapped_to']),
                'field'     => $field
            );

            if ($variable['mapped_to'] === 'TEXT_FIELD') {
                $mappedVariable['value'] = sanitize_text_field($variable['value']);
            }

            $variableMapping[] = $mappedVariable;
        }

        $configData = array(
            'campaignData' => array(
                'campaignName' => $campaignName,
                'startDate'    => '2020-01-01',
                'endDate'      => null
            ),
            'printNodes' => array(
                array(
                    'printNodeId' => 'Print_Node_01',
                    'description' => 'Print_Node_01'
                )
            ),
            'variables'  => $variables
        );

        $response = CrystalLiftService::instance()->userCampaignUpdateAction(
            get_option('stina_node_unique_id'),
            $campaignUniqueId,
            'f3b1a391-7df1-4ec4-b87c-5dd4395aff4a',
            $configData
        );

        if ($response) {
            Alert::success(__('Campaign updated successfully.', 'stina-crystal-lift'));
            crysli_update_campaign_meta($campaignUniqueId, 'variable_mapping', $variableMapping);
        } else {
            Alert::danger(__('There was an error while updating your campaign.', 'stina-crystal-lift'));
        }

        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $campaignUniqueId));
        exit();
    }

    /**
     * Updates the campaign's status.
     *
     * @return void
     * @throws Exception
     */
    public function updateCampaignStatus()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_update_status')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        $campaignUniqueId = sanitize_text_field($_POST['campaign_unique_id']);
        $campaignDescLong = sanitize_text_field($_POST['campaign_name']);
        $campaignType     = sanitize_text_field($_POST['campaign_type']);
        $campaignStatus   = sanitize_text_field($_POST['campaign_status']);
        $events = array(
            array(
                'eventUniqueId' => sanitize_text_field($_POST['campaign_event_unique_id'])
            )
        );
        $actions = array(
            array(
                'actionUniqueId' => sanitize_text_field($_POST['campaign_action_unique_id'])
            )
        );

        CrystalLiftService::instance()->userCampaignUpdateCampaign(
            $campaignUniqueId,
            $campaignDescLong,
            $campaignType,
            $campaignStatus,
            $events,
            $actions
        );

        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $campaignUniqueId));
        exit();
    }

    /**
     * Updates the campaign's event configuration
     *
     * @return void
     * @throws GuzzleException
     */
    public function updateCampaignEvent()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_update_event')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        $campaignEventConditions = $_POST['campaign_event_conditions'];
        $campaignUniqueId        = sanitize_text_field($_POST['campaign_unique_id']);
        $campaignDescLong        = sanitize_text_field($_POST['campaign_desc_long']);
        $campaignType            = sanitize_text_field($_POST['campaign_type']);
        $campaignStatus          = sanitize_text_field($_POST['campaign_status']);
        $eventObject             = sanitize_text_field($_POST['campaign_event_object']);
        $eventTrigger            = sanitize_text_field($_POST['campaign_event_trigger']);

        $campaignUpdate = CrystalLiftService::instance()->userCampaignUpdateCampaign(
            $campaignUniqueId,
            $campaignDescLong,
            $campaignType,
            $campaignStatus,
            array(
                array(
                    'eventUniqueId' => crysli_get_event_unique_id($eventObject, $eventTrigger)
                )
            ),
            array(
                array(
                    'actionUniqueId' => 'f3b1a391-7df1-4ec4-b87c-5dd4395aff4a'
                )
            )
        );

        if (!$campaignUpdate) {
            Alert::danger(__('There was an error while updating your campaign.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        if (!$campaignEventConditions) {
            $campaignEventConditions = array();
        }

        $campaignEventConfig = array(
            'object'     => sanitize_text_field($_POST['campaign_event_object']),
            'trigger'    => sanitize_text_field($_POST['campaign_event_trigger']),
            'conditions' => $campaignEventConditions
        );

        crysli_update_campaign_meta($campaignUniqueId, 'event_configuration', $campaignEventConfig);

        Alert::success(__('Campaign event configured successfully.', 'stina-crystal-lift'));
        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $campaignUniqueId));
        exit();
    }

    /**
     * Updates the campaign's recipients
     *
     * @return void
     */
    public function updateCampaignRecipients()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_update_recipients')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        $conditions         = array();
        $campaignUniqueId   = sanitize_text_field($_POST['campaign_unique_id']);
        $campaignRecipients = $_POST['campaign_recipients'];

        // We clear the previously selected recipients to avoid duplicate entries
        CampaignRecipientRelResourceRepository::instance()->deleteBy(
            array(
                'campaign_unique_id' => $campaignUniqueId
            )
        );

        // Add the newly selected recipients
        foreach ($campaignRecipients as $userId) {
            CampaignRecipientRelResourceRepository::instance()->insert(
                array(
                    'campaign_unique_id' => $campaignUniqueId,
                    'user_id'            => $userId
                )
            );
        }

        // Update the campaign recipient conditions
        if (isset($_POST['campaign_condition'])) {
            foreach ($_POST['campaign_condition'] as $condition) {
                if (is_array($condition['value'])) {
                    $value = array();

                    foreach ($condition['value'] as $conditionValue) {
                        $value[] = sanitize_text_field($conditionValue);
                    }
                } else {
                    $value = sanitize_text_field($condition['value']);
                }

                $conditions[] = array(
                    'object'   => sanitize_text_field($condition['object']),
                    'entity'   => sanitize_text_field($condition['entity']),
                    'operator' => sanitize_text_field($condition['operator']),
                    'value'    => $value
                );
            }
        }

        crysli_update_campaign_meta($campaignUniqueId, 'recipient_conditions', $conditions);

        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $campaignUniqueId));
        exit();
    }

    public function updateTriggerDialogConfirmation()
    {
        if (!wp_verify_nonce($_POST['_wpnonce'], 'crysli_campaign_update_trigger_dialog_confirmation')) {
            Alert::danger(__('Invalid nonce.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&action=signup'));
            exit;
        }

        $campaignUniqueId = sanitize_text_field($_POST['campaign_unique_id']);

        crysli_update_campaign_meta($campaignUniqueId, 'trigger_dialog_confirmation', 1);

        wp_redirect(admin_url('admin.php?page=' . PageEnumerator::CAMPAIGNS . '&id=' . $campaignUniqueId));
        exit();
    }

    /**
     * Loads the needed data for the "Create Campaign" wizard.
     *
     * @return void
     */
    public function campaignCreateWizardLoad()
    {
        check_ajax_referer('crysli_modal_campaign_create_wizard_load', 'security');

        $events = array();

        foreach (crysli_get_events() as $event) {
            $events[] = $event->toArray();
        }

        wp_send_json(
            array(
                'events' => $events
            ),
            200
        );
    }

    /**
     * Loads campaign's event configuration.
     *
     * @return void
     */
    public function campaignConfigureEventWizardLoad()
    {
        check_ajax_referer('crysli_modal_campaign_configure_event_wizard_load', 'security');

        $campaignUniqueId = sanitize_text_field($_POST['campaign_unique_id']);

        if (!$config = crysli_get_campaign_meta($campaignUniqueId, 'event_configuration', true)) {
            $config = Event::getBlankConfig();
        }

        wp_send_json(
            array(
                'config' => $config
            )
        );
    }

    /**
     * Loads the needed data for the "Edit Campaign Recipients" wizard.
     *
     * @return void
     */
    public function campaignEditRecipientsWizardLoad()
    {
        check_ajax_referer('crysli_modal_campaign_edit_recipients_wizard_load', 'security');

        $term          = sanitize_text_field($_POST['term']);
        $limit         = intval($_POST['limit']);
        $page          = intval($_POST['page']);
        $orderBy       = sanitize_text_field($_POST['orderBy']);
        $order         = sanitize_text_field($_POST['order']);
        $offset        = 0;
        $conditions    = json_decode(stripslashes($_POST['conditions']), true);
        $list          = array();
        $filteredUsers = array();

        if ($page !== 1) {
            $offset = ($page - 1) * $limit;
        }

        $args = array(
            'number'  => $limit,
            'orderby' => $orderBy,
            'order'   => $order
        );

        if ($offset > 0) {
            $args['offset'] = $offset;
        }

        if ($term && $term !== '') {
            $args['search'] = '*' . esc_attr($term) . '*';
            $args['search_columns'] = array('user_login', 'user_email', 'user_nicename', 'display_name');
        }

        $users = new WP_User_Query($args);

        if (!empty($conditions)) {
            foreach ($conditions as $condition) {
                $object   = sanitize_text_field($condition['object']);
                $entity   = sanitize_text_field($condition['entity']);
                $operator = sanitize_text_field($condition['operator']);
                $value    = $condition['value'];

                /** @var WP_User $user */
                foreach ($users->get_results() as $user) {
                    $user = get_user_by('ID', $user->ID);

                    switch ($object) {
                        case 'customer':
                            switch ($entity) {
                                case 'registration_date':
                                    try {
                                        $dateRegistered = new DateTime($user->user_registered);
                                        $dateFiltered   = new DateTime(sanitize_text_field($value));
                                    } catch (Exception $e) {
                                        break;
                                    }

                                    $dateRegistered->setTime(0, 0, 0, 0);
                                    $dateFiltered->setTime(0, 0, 0, 0);

                                    if ($this->evaluateCondition($dateRegistered, $dateFiltered, $operator)) {
                                        $filteredUsers[] = $user;
                                    }

                                    break;
                                case 'language':
                                case 'shipping_country':
                                case 'billing_country':
                                    $countries = array();

                                    foreach ($value as $country) {
                                        $countries[] = sanitize_text_field($country);
                                    }

                                    $userCountry = get_user_meta($user->ID, $entity, true);

                                    if ($this->evaluateCondition($userCountry, $countries, $operator)) {
                                        $filteredUsers[] = $user;
                                    }
                                case 'gender':
                                case 'birthday':
                            }
                            break;
                        case 'order':
                            switch ($entity) {
                                case 'first_order_date':
                                case 'last_order_date':
                                    $args = array(
                                        'customer_id' => $user->ID,
                                        'orderby'     => 'date',
                                        'order'       => ($entity === 'first_order_date') ? 'ASC' : 'DESC',
                                        'return'      => 'objects',
                                        'limit'       => 1
                                    );

                                    $orders = wc_get_orders($args);

                                    if (empty($orders) || count($orders) !== 1) {
                                        break;
                                    }

                                    /** @var WC_Order $order */
                                    $order = $orders[0];

                                    try {
                                        $dateOrdered  = $order->get_date_created();
                                        $dateFiltered = new DateTime(sanitize_text_field($value));
                                    } catch (Exception $e) {
                                        break;
                                    }

                                    $dateOrdered->setTime(0, 0, 0, 0);
                                    $dateFiltered->setTime(0, 0, 0, 0);

                                    if ($this->evaluateCondition($dateOrdered, $dateFiltered, $operator)) {
                                        $filteredUsers[] = $user;
                                    }

                                    break;
                                case 'number_of_orders':
                                    $args = array(
                                        'customer_id' => $user->ID,
                                        'limit'       => -1,
                                        'return'      => 'ids'
                                    );
                                    $orders = count(wc_get_orders($args));
                                    $value = intval($value);

                                    if ($this->evaluateCondition($orders, $value, $operator)) {
                                        $filteredUsers[] = $user;
                                    }

                                    break;
                                case 'revenue_completed':
                                    $args = array(
                                        'customer_id' => $user->ID,
                                        'status' => 'completed'
                                    );

                                    /** @var WC_Order[] $orders */
                                    $orders = wc_get_orders($args);
                                    $value = floatval($value);
                                    $revenue = 0;

                                    /** @var WC_Order $order */
                                    foreach ($orders as $order) {
                                        $revenue += $order->get_total();
                                    }

                                    if ($this->evaluateCondition($revenue, $value, $operator)) {
                                        $filteredUsers[] = $user;
                                    }

                                    break;
                            }
                            break;
                    }
                }
            }
        } else {
            $filteredUsers = $users->get_results();
        }

        /** @var WP_User $user */
        foreach ($filteredUsers as $user) {
            $list[] = array(
                'id'        => $user->ID,
                'firstName' => get_user_meta($user->ID, 'first_name', true),
                'lastName'  => get_user_meta($user->ID, 'last_name', true),
                'email'     => $user->user_email
            );
        }

        wp_send_json(
            array(
                'total' => count($filteredUsers),
                'list'  => $list
            )
        );
    }

    /**
     * @param $value1
     * @param $value2
     * @param string $operator
     * @return bool
     */
    private function evaluateCondition($value1, $value2, $operator)
    {
        switch ($operator) {
            case 'LT':
                return ($value1 < $value2);
            case 'LE':
                return ($value1 <= $value2);
            case 'GT':
                return ($value1 > $value2);
            case 'GE':
                return ($value1 >= $value2);
            case 'EQ':
                return ($value1 === $value2);
            case 'IN':
                return (in_array($value1, $value2));
            case 'NIN':
                return (!in_array($value1, $value2));
        }
    }
}