<?php


namespace STINA\CrystalLift\Components;

use STINA\CrystalLift\Enumerators\Page as PageEnumerator;
use STINA\CrystalLift\Services\CrystalLift as CrystalLiftService;
use STINA\Library\Components\UI\Heading;
use STINA\Library\Interfaces\Component as ComponentInterface;
use STINA\Library\Interfaces\UiComponent as UiComponentInterface;
use STINA\Library\Models\User as UserModel;
use STINA\Library\Repositories\User as UserRepository;

defined('ABSPATH') || exit;

class Settings implements ComponentInterface
{
    /**
     * @var false|UserModel
     */
    protected $user;

    /**
     * @var UiComponentInterface[]
     */
    protected $ui;

    /**
     * @var array
     */
    protected $environments;

    /**
     * Settings constructor.
     */
    public function __construct()
    {
        $this->user = UserRepository::instance()->get();

        // UI Components
        $this->ui = array(
            'heading' => new Heading(
                __('Settings', 'stina-crystal-lift'),
                __('Sign out', 'stina-crystal-lift'),
                $this->user,
                'crysli',
                array(
                    'redirect' => '/wp-admin/admin.php?page=' . PageEnumerator::DASHBOARD
                )
            )
        );

        $this->environments = get_option('crysli_environments', array());

        add_action('admin_menu',                   array($this, 'init'));
        add_action('wp_ajax_crysli_settings_save', array($this, 'save'));
        add_action('wp_ajax_crysli_settings_load', array($this, 'load'));

        // Sections
        add_action('wp_ajax_crysli_settings_load_section_services', array($this, 'loadSectionServices'));
        add_action('wp_ajax_crysli_settings_load_user_profile',     array($this, 'loadUserProfile'));

        // Other actions
        add_action('wp_ajax_crysli_settings_load_log_file', array($this, 'loadLogFile'));
    }

    /**
     * @inheritDoc
     */
    public function init()
    {
        if (!$this->user) {
            return;
        }

        add_submenu_page(
            'crystal-lift',
            __('Settings', 'stina-crystal-lift'),
            __('Settings', 'stina-crystal-lift'),
            'manage_options',
            PageEnumerator::SETTINGS,
            array($this, 'showPage')
        );
    }

    /**
     * @inheritDoc
     */
    public function showPage()
    {
        // Enqueue needed scripts and styles
        wp_enqueue_style('crysli-page-settings');
        wp_enqueue_script('crysli-component-settings');
        wp_enqueue_script('stina-framework');
        wp_enqueue_script('stina-fontawesome');

        // Localize scripts and pass needed variables
        wp_localize_script(
            'crysli-component-settings',
            'security',
            array(
                'save'                => wp_create_nonce('crysli_settings_save'),
                'load'                => wp_create_nonce('crysli_settings_load'),
                'loadSectionServices' => wp_create_nonce('crysli_settings_load_section_services'),
                'loadUserProfile'     => wp_create_nonce('crysli_settings_load_user_profile')
            )
        );

        wp_localize_script(
            'crysli-component-settings',
            'i18n',
            array()
        );

        // Load templates
        crysli_get_template(
            'page-settings/component-main.php',
            array(
                'environments' => $this->environments,
                'environment'  => get_option('crysli_environment', ''),
                'ui'           => $this->ui,
                'logFiles'     => array_diff(scandir(CRYSLI_LOG_DIR), array('.', '..'))
            )
        );
    }

    /**
     * Saves the current settings.
     *
     * @return void
     */
    public function save()
    {
        check_ajax_referer('crysli_settings_save', 'security');

        $environment = sanitize_text_field($_POST['system_general_environment']);
        $logLevel    = sanitize_text_field($_POST['system_general_logLevel']);

        // System
        update_option('crysli_environment',      $environment);
        update_option('crysli_application_mode', 'LIVE');
        update_option('crysli_log_level',        $logLevel);

        wp_send_json(1, 200);
        exit;
    }

    /**
     * Loads the current settings.
     *
     * @return void
     */
    public function load()
    {
        $settings = array(
            'services' => array(
                'deutschePostTriggerDialog' => array()
            ),
            'tools' => array(),
            'system' => array(
                'general' => array(
                    'environment'     => get_option('crysli_environment', ''),
                    'applicationMode' => 'LIVE',
                    'logLevel'        => get_option('crysli_log_level', 400)
                )
            ),
            'support' => array()
        );

        wp_send_json($settings, 200);
        exit;
    }

    /**
     * Loads the services section page data.
     *
     * @return void
     */
    public function loadSectionServices()
    {
        check_ajax_referer('crysli_settings_load_section_services', 'security');

        $userProfile = CrystalLiftService::instance()->userUserGetProfile();

        if ($userProfile) {
            wp_send_json(
                array(
                    'username'     => $userProfile['data']['username'],
                    'email'        => $userProfile['data']['email'],
                    'firstName'    => $userProfile['data']['firstName'],
                    'lastName'     => $userProfile['data']['lastName'],
                    'masClientId'  => $userProfile['data']['id'],
                    'subscription' => 'Starter'
                ),
                200
            );
        } else {
            wp_send_json_error();
        }
    }


    /**
     * Loads the user's profile data.
     *
     * @return void
     */
    public function loadUserProfile()
    {
        check_ajax_referer('crysli_settings_load_user_profile', 'security');

        $userProfile = CrystalLiftService::instance()->userUserGetProfile();

        if ($userProfile) {
            wp_send_json($userProfile['data'], 200);
        } else {
            wp_send_json_error();
        }
    }

    /**
     * Loads the requested log file.
     *
     * @return void
     */
    public function loadLogFile()
    {
        $file = $_POST['file'];

        $logFileContent = file_get_contents(CRYSLI_LOG_DIR . $file);

        wp_send_json($logFileContent, 200);
    }
}