<?php


namespace STINA\CrystalLift\Controllers;

use GuzzleHttp\Exception\GuzzleException;
use STINA\CrystalLift\Enumerators\CampaignEvent as CampaignEventEnumerator;
use STINA\CrystalLift\Enumerators\CampaignStatus as CampaignStatusEnumerator;
use STINA\CrystalLift\Enumerators\CampaignType as CampaignTypeEnumerator;
use STINA\CrystalLift\Events\Manager as EventManager;
use STINA\CrystalLift\Services\CrystalLift as CrystalLiftService;
use STINA\Library\Repositories\User;
use WC_Order;
use WP_User;

defined('ABSPATH') || exit;

class Event
{
    /**
     * Event constructor.
     */
    public function __construct()
    {
        // EVENT: CUSTOMER_REGISTER
        add_action('user_register', array($this, 'eventCustomerRegister'));

        // EVENT: ORDER_PLACED
        add_action('woocommerce_new_order', array($this, 'eventOrderPlaced'));
        add_action('woocommerce_thankyou',  array($this, 'eventOrderPlaced'));
    }

    /**
     * CUSTOMER_REGISTER event handler.
     *
     * @param int $userId
     * @throws GuzzleException
     */
    public function eventCustomerRegister($userId)
    {
        // The user is not logged in, do nothing
        if ($user = User::instance()->get() === false) {
            return;
        }

        /** @var bool|WP_User $user */
        $user = get_user_by('ID', $userId);

        // Only continue if the user exists and if he is a customer
        if (!$user || !in_array('customer', $user->roles)) {
            return;
        }

        // Retrieve all active automated campaigns
        $campaigns = CrystalLiftService::instance()->userCampaignListCampaign(
            1,
            1000000,
            array(),
            array(
                array(
                    'key' => 'campaignStatus',
                    'value' => CampaignStatusEnumerator::CAMPAIGN_ACTIVE
                ),
                array(
                    'key' => 'campaignType',
                    'value' => CampaignTypeEnumerator::CAMPAIGN_AUTOMATED_EVENT_BASED
                )
            ),
            null
        );

        // There are no active automated campaigns
        if (!$campaigns) {
            return;
        }

        foreach ($campaigns['data']['list'] as $campaign) {
            $campaignUniqueId      = $campaign['campaignUniqueId'];
            $campaignEventUniqueId = $campaign['events'][0]['eventUniqueId'];

            // The event unique ID does not match
            if ($campaignEventUniqueId !== CampaignEventEnumerator::CUSTOMER_REGISTERED) {
                continue;
            }

            // Event hasn't been configured
            if (!EventManager::isCampaignEventConfigured($campaignUniqueId)) {
                continue;
            }

            $eventConfig = crysli_get_campaign_meta($campaignUniqueId, 'event_configuration', true);

            $args = array('user_id' => $userId);

            // Event conditions don't match
            if (!EventManager::evaluateConditions($eventConfig, $args)) {
                continue;
            }

            // Everything matched, trigger!
            EventManager::trigger($campaign, (array)$userId);
        }
    }

    /**
     * ORDER_PLACED event handler.
     *
     * @param int $orderId
     * @throws GuzzleException
     */
    public function eventOrderPlaced($orderId)
    {
        // The user is not logged in, do nothing.
        if ($user = User::instance()->get() === false) {
            return;
        }

        /** @var false|WC_Order $order */
        $order = wc_get_order($orderId);

        if (!$order) {
            return;
        }

        // Retrieve all automated, active campaigns
        $campaigns = CrystalLiftService::instance()->userCampaignListCampaign(
            1,
            1000000,
            array(),
            array(
                array(
                    'key' => 'campaignStatus',
                    'value' => CampaignStatusEnumerator::CAMPAIGN_ACTIVE
                ),
                array(
                    'key' => 'campaignType',
                    'value' => CampaignTypeEnumerator::CAMPAIGN_AUTOMATED_EVENT_BASED
                )
            ),
            null
        );

        if (!$campaigns) {
            return;
        }

        foreach ($campaigns['data']['list'] as $campaign) {
            $campaignUniqueId      = $campaign['campaignUniqueId'];
            $campaignEventUniqueId = $campaign['events'][0]['eventUniqueId'];

            // The event unique ID does not match
            if ($campaignEventUniqueId !== CampaignEventEnumerator::ORDER_PLACED) {
                continue;
            }

            // Event hasn't been configured
            if (!EventManager::isCampaignEventConfigured($campaignUniqueId)) {
                continue;
            }

            $eventConfig = crysli_get_campaign_meta($campaignUniqueId, 'event_configuration', true);

            $args = array('order_id' => $orderId);

            // Event conditions don't match
            if (!EventManager::evaluateConditions($eventConfig, $args)) {
                continue;
            }

            // Everything matched, trigger!
            EventManager::trigger($campaign, (array)$order->get_customer_id());
        }
    }

}