<?php


namespace STINA\CrystalLift\Events;

use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\ServerException;
use STINA\CrystalLift\Events\WooCommerce\CustomerRegistered;
use STINA\CrystalLift\Events\WooCommerce\OrderPlaced;
use STINA\CrystalLift\Interfaces\Event as EventInterface;
use STINA\CrystalLift\Logger;
use STINA\CrystalLift\Services\CrystalLift as CrystalLiftService;
use STINA\CrystalLift\Repositories\Resources\CampaignTrigger as CampaignTriggerResourceRepository;

defined('ABSPATH') || exit;

class Manager
{
    /**
     * Finds and returns the event by it's unique identifier.
     *
     * @param string $eventUniqueId
     * @return false|EventInterface
     */
    public static function getEvent($eventUniqueId)
    {
        $mapping = array(
            CustomerRegistered::UNIQUE_ID => CustomerRegistered::class,
            OrderPlaced::UNIQUE_ID => OrderPlaced::class
        );

        if (!isset($mapping[$eventUniqueId]) || !class_exists($mapping[$eventUniqueId])) {
            return false;
        }

        /** @var EventInterface $event */
        $event = new $mapping[$eventUniqueId];

        return $event;
    }

    /**
     * Determines whether a campaign has a valid event configuration.
     *
     * @param string $campaignUniqueId
     * @return bool
     */
    public static function isCampaignEventConfigured($campaignUniqueId)
    {
        $config = crysli_get_campaign_meta($campaignUniqueId, 'event_configuration', true);

        if (!$config || !isset($config['object']) || !isset($config['trigger']) || !isset($config['conditions'])) {
            return false;
        }

        if (is_array($config['conditions']) && count($config['conditions']) > 0) {
            foreach ($config['conditions'] as $condition) {
                if (!$condition['value']) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Determines whether the campaign event conditions match or not.
     *
     * @param array $config
     * @param array $args
     * @return bool
     */
    public static function evaluateConditions($config, $args)
    {
        $conditions = $config['conditions'];

        // Return early if there are no conditions defined.
        if (empty($conditions)) {
            return true;
        }

        // Iterate through conditions and check them on an individual level
        foreach ($conditions as $condition) {
            $comparisonValue = crysli_get_condition_entity_value($condition['entity'], $args);

            if ($comparisonValue === false) {
                return false;
            }

            if (empty(array_intersect($comparisonValue, $condition['value']))) {
                // No intersection, values aren't present
                switch ($condition['operator']) {
                    case 'IN':
                        return false;
                    case 'NIN':
                        return true;
                }
            } else {
                // Intersection, values are present
                switch ($condition['operator']) {
                    case 'IN':
                        return true;
                    case 'NIN':
                        return false;
                }
            }
        }

        return false;
    }

    /**
     * Triggers a campaign with the given array of user IDs.
     *
     * @param array $campaign
     * @param array $userIds
     * @throws GuzzleException
     */
    public static function trigger($campaign, $userIds)
    {
        $logger                = Logger::instance();
        $response              = null;
        $campaignUniqueId      = $campaign['campaignUniqueId'];
        $campaignEventUniqueId = $campaign['events'][0]['eventUniqueId'];
        $campaignData          = $campaign['actions'][0]['config']['campaignData'];
        $campaignStartDate     = stina_get_utc_date_time($campaignData['startDate']);
        $campaignEndDate       = null;
        $now                   = stina_get_utc_date_time();

        $logger->setLogger(Logger::LOG_EVENTS);

        if ($campaignData['endDate'] !== null) {
            $campaignEndDate = stina_get_utc_date_time($campaignData['endDate']);
        }

        // Campaign has not yet begun
        if ($now < $campaignStartDate) {
            $logger->warning(
                "Campaign '{$campaign['campaignDescLong']}' did not trigger. Reason: Campaign has not begun yet.",
                $campaign
            );
            return;
        }

        // Campaign has ended
        if ($campaignEndDate !== null && $now > $campaignEndDate) {
            $logger->warning(
                "Campaign '{$campaign['campaignDescLong']}' did not trigger. Reason: Campaign has ended.",
                $campaign
            );
            return;
        }

        $campaignVariableMapping = crysli_get_campaign_meta($campaignUniqueId, 'variable_mapping', true);

        // The variable mapping was not found
        if (!$campaignVariableMapping) {
            $logger->warning(
                "Campaign '{$campaign['campaignDescLong']}' did not trigger. Reason: Variable mapping was not found.",
                $campaign
            );
            return;
        }

        $campaignVariables = array();
        $campaignVariableTriggers = array();

        foreach ($userIds as $userId) {
            $userMappedVariables = crysli_map_variables_to_user($userId, $campaignUniqueId, $campaignVariableMapping);

            // The variables couldn't be mapped to the user
            if (!$userMappedVariables) {
                $logger->warning(
                    "User with ID: '{$userId}' for campaign '{$campaign['campaignDescLong']}' did not trigger. Reason: Missing user data from the variable mapping.",
                    $campaign
                );
                continue;
            }

            $campaignVariables[] = $userMappedVariables;
            $campaignVariableTriggers[$userId] = $userMappedVariables;
        }

        // Not a single user has mapped variables
        if (empty($campaignVariables)) {
            $logger->warning(
                "Campaign '{$campaign['campaignDescLong']}' did not trigger. Reason: Not a single user has mapped variables.",
                $campaign
            );
            return;
        }

        try {
            $response = CrystalLiftService::instance()->userEventTriggerCampaign(
                get_option('stina_node_unique_id'),
                $campaignUniqueId,
                $campaignEventUniqueId,
                array(
                    'campaignVariables' => $campaignVariables,
                    'printNode' => 'Print_Node_01'
                )
            );
        } catch (ServerException $se) {
            $logger->setLogger(Logger::LOG_PLUGIN);
            $logger->error($se->getMessage(), $se->getTrace());
        }

        $logger->setLogger(Logger::LOG_EVENTS);

        if ($response) {
            $logger->info(
                "Campaign '{$campaign['campaignDescLong']}' triggered successfully.",
                $campaign
            );

            foreach ($userIds as $userId) {
                if (isset($campaignVariableTriggers[$userId])) {
                    CampaignTriggerResourceRepository::instance()->insert(
                        array(
                            'campaign_unique_id' => $campaignUniqueId,
                            'trigger_unique_id'  => $campaignVariableTriggers[$userId]['triggerUniqueId'],
                            'variable_mapping'   => serialize($campaignVariableTriggers[$userId]),
                            'user_id'            => $userId
                        )
                    );
                }
            }
        } else {
            $logger->error(
                "Campaign {$campaign['campaignDescLong']} failed to trigger.",
                $campaign
            );
        }
    }
}