<?php


namespace STINA\CrystalLift;

use Exception;
use Monolog\Handler\RotatingFileHandler;
use Monolog\Logger as MonologLogger;
use Psr\Log\LoggerInterface;
use STINA\Library\Abstracts\Singleton;

defined('ABSPATH') || exit;

class Logger extends Singleton implements LoggerInterface
{
    /**
     * @var string
     */
    const LOG_PLUGIN = 'crystal_lift.log';

    /**
     * @var string
     */
    const LOG_API = 'crystal_lift_api.log';

    /**
     * @var string
     */
    const LOG_EVENTS = 'crystal_lift_events.log';

    /**
     * @var MonologLogger
     */
    protected $logger;

    /**
     * @var MonologLogger
     */
    protected $pluginLogger;

    /**
     * @var MonologLogger
     */
    protected $apiLogger;

    /**
     * @var MonologLogger
     */
    protected $eventLogger;

    /**
     * Logger constructor.
     */
    public function __construct()
    {
        // Set up the logger instances
        $this->pluginLogger = new MonologLogger('PLUGIN');
        $this->apiLogger    = new MonologLogger('API');
        $this->eventLogger  = new MonologLogger('EVENT');

        // Push the needed handlers
        try {
            $this->pluginLogger->pushHandler(
                new RotatingFileHandler(
                    CRYSLI_LOG_DIR . self::LOG_PLUGIN,
                    30,
                    MonologLogger::DEBUG
                )
            );

            $this->apiLogger->pushHandler(
                new RotatingFileHandler(
                    CRYSLI_LOG_DIR . self::LOG_API,
                    30,
                    MonologLogger::NOTICE
                )
            );

            $this->eventLogger->pushHandler(
                new RotatingFileHandler(
                    CRYSLI_LOG_DIR . self::LOG_EVENTS,
                    30,
                    MonologLogger::DEBUG
                )
            );
        } catch (Exception $e) {
        }
    }

    /**
     * Sets the logger that needs to be used.
     *
     * @param $file
     */
    public function setLogger($file)
    {
        switch ($file) {
            case self::LOG_PLUGIN:
            default:
                $this->logger = $this->pluginLogger;
                break;
            case self::LOG_API:
                $this->logger = $this->apiLogger;
                break;
            case self::LOG_EVENTS:
                $this->logger = $this->eventLogger;
                break;
        }
    }

    /**
     * System is unusable.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function emergency($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::EMERGENCY, $message, $context);
    }

    /**
     * Action must be taken immediately.
     *
     * Example: Entire website down, database unavailable, etc. This should
     * trigger the SMS alerts and wake you up.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function alert($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::ALERT, $message, $context);
    }

    /**
     * Critical conditions.
     *
     * Example: Application component unavailable, unexpected exception.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function critical($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::CRITICAL, $message, $context);
    }

    /**
     * Runtime errors that do not require immediate action but should typically
     * be logged and monitored.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function error($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::ERROR, $message, $context);
    }

    /**
     * Exceptional occurrences that are not errors.
     *
     * Example: Use of deprecated APIs, poor use of an API, undesirable things
     * that are not necessarily wrong.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function warning($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::WARNING, $message, $context);
    }

    /**
     * Normal but significant events.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function notice($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::NOTICE, $message, $context);
    }

    /**
     * Interesting events.
     *
     * Example: User logs in, SQL logs.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function info($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::INFO, $message, $context);
    }

    /**
     * Detailed debug information.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function debug($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::DEBUG, $message, $context);
    }

    /**
     * Logs with an arbitrary level.
     *
     * @param int    $level   The log level
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function log($level, $message, array $context = array())
    {
        $level = MonologLogger::toMonologLevel($level);

        return $this->logger->addRecord($level, $message, $context);
    }
}