<?php


namespace STINA\CrystalLift\Services;

use Exception;
use GuzzleHttp\Exception\GuzzleException;
use STINA\CrystalLift\Abstracts\Service as AbstractService;
use STINA\CrystalLift\Enumerators\Feature;
use STINA\CrystalLift\Enumerators\Page as PageEnumerator;
use STINA\CrystalLift\Logger;
use STINA\CrystalLift\Services\CrystalLift as CrystalLiftService;
use STINA\Library\Alert;
use STINA\Library\Models\User as UserModel;
use STINA\Library\Repositories\User as UserRepository;
use STINA\Library\Connector;
use STINA\Library\Uuid;

defined('ABSPATH') || exit;

class CrystalLift extends AbstractService
{
    /**
     * CrystalLift constructor.
     */
    public function __construct()
    {
        $baseUri = get_option('crysli_environment', false);

        if ($baseUri === false) {
            throw new Exception('Node is not configured.');
        }

        $this->connector = new Connector($baseUri, 'p00057', 'v1');
        $this->logger = Logger::instance();
        $this->logger->setLogger(Logger::LOG_API);
    }

    /**
     * Initializes the open node with the remote service.
     *
     * @return bool|array
     * @throws GuzzleException
     */
    public function openNodeInitializeNode()
    {
        $infoObject = $this->connector->createInfoObject(
            'OPEN_NODE_INITIALIZE_NODE',
            null,
            '',
            array(
                'appUniqueId'                  => Uuid::v4(),
                'nodeUniqueId'                 => Uuid::v4(),
                'nodePortfolioElementUniqueId' => CRYSLI_NODE_PORTFOLIO_ELEMENT_UNIQUE_ID,
                'nodeType'                     => 'NODE_PLUGIN_WORDPRESS',
                'hostSystem'                   => 'WordPress 5.4.1',
                'hostUrl'                      => get_site_url(),
                'hostRegion'                   => 'DE',
                'userLocale'                   => get_user_locale(),
                'userRegion'                   => 'AT',
                'clientRegion'                 => 'DE'
            )
        );

        return $this->request($infoObject);
    }

    /**
     * Retrieves a JWT for authentication for all PUBLIC requests. The token is
     * stored as a transient.
     *
     * @param string $nodeUniqueId
     * @return bool|array
     * @throws GuzzleException
     */
    public function openNodeLoginNode($nodeUniqueId)
    {
        $infoObject = $this->connector->createInfoObject(
            'OPEN_NODE_LOGIN_NODE',
            null,
            '',
            array(
                'nodeUniqueId'                 => $nodeUniqueId,
                'nodePortfolioElementUniqueId' => '04d5b79b-0a66-4b69-991d-254f91e409a5'
            )
        );

        $response = $this->request($infoObject);

        if (!$response) {
            return false;
        }

        $this->setNodeApiCredentials($response['data']['key'], $response['data']['secret']);

        return $response;
    }

    /**
     * Retrieves a JWT for authentication for all USER requests. The token is
     * stored as a transient.
     *
     * @param string $email
     * @param string $password
     * @param string $nodeUniqueId
     * @return bool|array
     * @throws Exception|GuzzleException
     */
    public function publicUserLoginUser($email, $password, $nodeUniqueId)
    {
        $infoObject = $this->connector->createInfoObject(
            'PUBLIC_USER_LOGIN_USER',
            Feature::GENERAL,
            '',
            array(
                'nodeUniqueId' => $nodeUniqueId,
                'username'     => $email,
                'password'     => $password
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getNodeBasicAuthToken());

        if (!$response) {
            return false;
        }

        $this->setUserApiCredentials($response['data']['key'], $response['data']['secret']);

        return $response;
    }

    /**
     * Retrieves the currently logged in user profile.
     *
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userUserGetProfile()
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_USER_GET_PROFILE',
            Feature::GENERAL,
            '',
            array()
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Creates a single campaign.
     *
     * @param string $feature
     * @param string $nodeUniqueId
     * @param string $campaignDescLong
     * @param string $campaignType
     * @param string $campaignReuseCampaignUniqueId
     * @param array  $events
     * @param array  $actions
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userCampaignCreateCampaign($feature, $nodeUniqueId, $campaignDescLong, $campaignType, $campaignReuseCampaignUniqueId, $events, $actions)
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_CAMPAIGN_CREATE_CAMPAIGN',
            $feature,
            '',
            array(
                'nodeUniqueId'                  => $nodeUniqueId,
                'campaignDescLong'              => $campaignDescLong,
                'campaignType'                  => $campaignType,
                'campaignReuseCampaignUniqueId' => $campaignReuseCampaignUniqueId,
                'events'                        => $events,
                'actions'                       => $actions
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Retrieves a single campaign.
     *
     * @param string $campaignUniqueId
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userCampaignGetCampaign($campaignUniqueId)
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_CAMPAIGN_GET_CAMPAIGN',
            Feature::GENERAL,
            '',
            array(
                'campaignUniqueId' => $campaignUniqueId
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Updates a single campaign.
     *
     * @param string $campaignUniqueId
     * @param string $campaignDescLong
     * @param string $campaignType
     * @param string $campaignStatus
     * @param array  $events
     * @param array  $actions
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userCampaignUpdateCampaign($campaignUniqueId, $campaignDescLong, $campaignType, $campaignStatus, $events, $actions)
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_CAMPAIGN_UPDATE_CAMPAIGN',
            Feature::GENERAL,
            '',
            array(
                'campaignUniqueId' => $campaignUniqueId,
                'campaignDescLong' => $campaignDescLong,
                'campaignType' => $campaignType,
                'campaignStatus' => $campaignStatus,
                'events' => $events,
                'actions' => $actions
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Retrieves multiple campaigns, as defined by the other parameters.
     *
     * @param int $page
     * @param int $limit
     * @param array $sort
     * @param array $filter
     * @param null|string $term
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userCampaignListCampaign($page = 1, $limit = 10, $sort = array(), $filter = array(), $term = null)
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_CAMPAIGN_LIST_CAMPAIGN',
            Feature::GENERAL,
            '',
            array(
                'page'   => $page,
                'limit'  => $limit,
                'sort'   => $sort,
                'filter' => $filter,
                'term'   => $term
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Updates the action and its data which is associated to the campaign.
     *
     * @param string $nodeUniqueId
     * @param string $campaignUniqueId
     * @param string $actionUniqueId
     * @param array  $configData
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userCampaignUpdateAction($nodeUniqueId, $campaignUniqueId, $actionUniqueId, $configData)
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_CAMPAIGN_UPDATE_ACTION',
            Feature::GENERAL,
            '',
            array(
                'nodeUniqueId' => $nodeUniqueId,
                'campaignUniqueId' => $campaignUniqueId,
                'actionUniqueId' => $actionUniqueId,
                'configData' => $configData
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Retrieves a filtered list of triggers for a specific campaign.
     *
     * @param string $campaignUniqueId
     * @param array  $filter
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userCampaignListTrigger($campaignUniqueId, $filter)
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_CAMPAIGN_LIST_TRIGGER',
            Feature::GENERAL,
            '',
            array(
                'campaignUniqueId' => $campaignUniqueId,
                'filter' => $filter
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Triggers the remote campaign.
     *
     * @param string $nodeUniqueId
     * @param string $campaignUniqueId
     * @param string $eventUniqueId
     * @param array  $actionData
     * @return false|array
     * @throws Exception|GuzzleException
     */
    public function userEventTriggerCampaign($nodeUniqueId, $campaignUniqueId, $eventUniqueId, $actionData)
    {
        $infoObject = $this->connector->createInfoObject(
            'USER_EVENT_TRIGGER_CAMPAIGN',
            Feature::GENERAL,
            '',
            array(
                'nodeUniqueId' => $nodeUniqueId,
                'campaignUniqueId' => $campaignUniqueId,
                'eventUniqueId' => $eventUniqueId,
                'actionData' => $actionData
            )
        );

        $response = $this->request($infoObject, 'Basic', $this->getUserBasicAuthToken());

        if (!$response) {
            return false;
        }

        return $response;
    }

    /**
     * Sets the node authentication token. The token value should always be
     * retrieved using the getNodeToken() method.
     *
     * @param string $token
     * @param int    $expiration
     * @return bool
     */
    public function setNodeToken($token, $expiration = 300)
    {
        return set_transient('crysli_token_node', $token, $expiration);
    }

    /**
     * Retrieves the node authentication token. If the token has expired, the
     * method will refresh the token.
     *
     * @return string
     * @throws Exception|GuzzleException
     */
    public function getNodeToken()
    {
        $token =  get_transient('crysli_token_node');

        if ($token) {
            return $token;
        }

        $nodeUniqueId = get_option('stina_node_unique_id', false);

        if (!$nodeUniqueId) {
            $this->openNodeInitializeNode();
            $nodeUniqueId = get_option('stina_node_unique_id', false);

            if ($nodeUniqueId === false) {
                throw new Exception('The Node UUID is missing from the system and cannot be initialized.');
            }
        }

        $this->openNodeLoginNode($nodeUniqueId);

        $token = get_transient('crysli_token_node');

        if (!$token) {
            throw new Exception('There was a problem while retrieving the public JWT token.');
        }

        return $token;
    }

    /**
     * Sets the user authentication token. The token value should always be
     * retrieved using the getUserToken() method.
     *
     * @param string $token
     * @param int    $expiration
     * @return bool
     */
    public function setUserToken($token, $expiration = 300)
    {
        return set_transient('stina_user_token', $token, $expiration);
    }

    /**
     * Retrieves the user authentication token. If the token has expired, the
     * method will refresh the token.
     *
     * @return mixed|string
     * @throws Exception|GuzzleException
     */
    public function getUserToken()
    {
        $token = get_transient('stina_user_token');

        if ($token) {
            return $token;
        }

        $nodeUniqueId = get_option('stina_node_unique_id', false);

        if (!$nodeUniqueId) {
            $this->openNodeInitializeNode();
            $nodeUniqueId = get_option('stina_node_unique_id', false);

            if ($nodeUniqueId === false) {
                throw new Exception('The Node UUID is missing from the system and cannot be initialized.');
            }
        }

        /** @var UserModel $user */
        $user = UserRepository::instance()->get();

        /** @var UserModel|bool $response */
        $response = CrystalLiftService::instance()->publicUserLoginUser(
            $user->getEmail(),
            $user->getPassword(),
            get_option('stina_node_unique_id')
        );

        if ($response === false) {
            // Delete the user credentials to force a new login.
            UserRepository::instance()->delete('crysli');
            Alert::danger(__('Your user credentials are invalid. Please login anew.', 'stina-crystal-lift'));
            wp_redirect(admin_url('admin.php?page=' . PageEnumerator::DASHBOARD));
            exit;
        }

        $token = $response['data']['token'];

        $this->setUserToken($token, 300);

        return $token;
    }

    /**
     * Stores the API key and secret for further service communication in the
     * plugin configuration.
     *
     * @param string $key
     * @param string $secret
     * @return bool
     */
    public function setNodeApiCredentials($key, $secret)
    {
        update_option('crysli_node_api_key', $key);
        update_option('crysli_node_api_secret', $secret);

        return true;
    }

    /**
     * Returns the API credentials prepared for use in the Basic authentication
     * header.
     *
     * @return string
     */
    public function getNodeBasicAuthToken()
    {
        $apiKey = get_option('crysli_node_api_key');
        $apiSecret = get_option('crysli_node_api_secret');

        return base64_encode($apiKey . ':' . $apiSecret);
    }

    /**
     * Stores the API key and secret for further service communication in the
     * plugin configuration.
     *
     * @param string $key
     * @param string $secret
     * @return bool
     */
    public function setUserApiCredentials($key, $secret)
    {
        update_option('crysli_user_api_key', $key);
        update_option('crysli_user_api_secret', $secret);

        return true;
    }

    /**
     * Returns the API credentials prepared for use in the Basic authentication
     * header.
     *
     * @return string
     */
    public function getUserBasicAuthToken()
    {
        $apiKey = get_option('crysli_user_api_key');
        $apiSecret = get_option('crysli_user_api_secret');

        return base64_encode($apiKey . ':' . $apiSecret);
    }
}