<?php


namespace STINA\Library\Components\UI;

use STINA\Library\Interfaces\UiComponent as UiComponentInterface;
use STINA\Library\Models\User as UserModel;
use STINA\Library\Repositories\User as UserRepository;

defined('ABSPATH') || exit;

class Heading implements UiComponentInterface
{
    /**
     * @var string
     */
    protected $html;

    /**
     * @var string
     */
    protected $title;

    /**
     * @var string
     */
    protected $signOutText;

    /**
     * @var false|UserModel
     */
    protected $user;

    /**
     * @var null|string
     */
    protected $plugin;

    /**
     * @var array
     */
    protected $options;

    /**
     * Heading constructor.
     *
     * @param string          $title
     * @param string          $signOutText
     * @param false|UserModel $user
     * @param null|string     $plugin
     * @param array           $options
     */
    public function __construct($title, $signOutText, $user = false, $plugin = null, $options = array())
    {
        add_action('admin_post_stina_user_sign_out', array($this, 'signOut'));

        $this->title = $title;
        $this->signOutText = $signOutText;
        $this->user = $user;
        $this->plugin = $plugin;

        $this->options = array_merge(
            array(
                'title'    => 'h1',
                'redirect' => null
            ),
            $options
        );

        $this->prepare();
    }

    /**
     * Signs the user out of his STINA account.
     *
     * @return void
     */
    public function signOut()
    {
        $nonce = wp_verify_nonce($_POST['_wpnonce'], 'stina_user_sign_out');

        if (!$nonce) {
            wp_die('Invalid nonce');
        }

        UserRepository::instance()->delete($this->plugin);

        $redirect = esc_url_raw($_POST['_wp_http_referer']);

        if ($this->options['redirect'] !== null) {
            $redirect = $this->options['redirect'];
        }

        wp_redirect(get_site_url() . $redirect);
        exit;
    }

    /**
     * Prepares the HTML for output.
     *
     * @return void
     */
    private function prepare()
    {
        $html = '';

        switch ($this->options['title']) {
            default:
            case 'h1':
                $headerTitle = '<h1 class="stina-header-title">' . $this->title . '</h1>';
                break;
            case 'h2':
                $headerTitle = '<h2 class="stina-header-title">' . $this->title . '</h2>';
                break;
            case 'h3':
                $headerTitle = '<h3 class="stina-header-title">' . $this->title . '</h3>';
                break;
            case 'h4':
                $headerTitle = '<h4 class="stina-header-title">' . $this->title . '</h4>';
                break;
            case 'h5':
                $headerTitle = '<h5 class="stina-header-title">' . $this->title . '</h5>';
                break;
            case 'h6':
                $headerTitle = '<h6 class="stina-header-title">' . $this->title . '</h6>';
                break;
        }

        $html .= '<div class="stina-header">';
        $html .= $headerTitle;

        if ($this->user !== false && $this->user instanceof UserModel) {
            $html .= '<div class="stina-header-action-bar">';
            $html .= '<form class="stina-text-right" action="' . esc_url(admin_url('admin-post.php')) . '" method="post">';
            $html .= '<input type="hidden" name="action" value="stina_user_sign_out">';
            $html .= wp_nonce_field('stina_user_sign_out', '_wpnonce', true, false);
            $html .= '<div class="stina-flexbox stina-mb-1">';
            $html .= '<p class="stina-m-0 stina-mr-1 stina-d-ib">';
            $html .= '<strong>' . $this->user->getFirstName() . ' ' . $this->user->getLastName() . '</strong>';
            $html .= '<br>';
            $html .= '<small>' . $this->user->getEmail() . '</small>';
            $html .= '</p>';
            $html .= '<img class="stina-d-ib" src="' . esc_url(stina_get_gravatar_image_url($this->user->getEmail(), 39)) . '">';
            $html .= '</div>';
            $html .= '<button type="submit" class="stina-btn stina-btn-primary">';
            $html .= $this->signOutText;
            $html .= '</button>';
            $html .= '</form>';
            $html .= '</div>';
        }

        $html .= '</div>';

        $this->html = $html;
    }

    /**
     * Renders the HTML of the component.
     *
     * @param bool $echo
     * @return void|string
     */
    public function render($echo = true)
    {
        if ($echo === false) {
            return $this->html;
        }

        echo $this->html;
    }
}